package com.agilecontrol.nea.core.report;

import com.agilecontrol.nea.core.control.web.SessionContextManager;
import com.agilecontrol.nea.core.control.web.UserWebImpl;
import com.agilecontrol.nea.core.control.web.WebUtils;
import com.agilecontrol.nea.core.query.QueryEngine;
import com.agilecontrol.nea.core.query.QueryRequest;
import com.agilecontrol.nea.core.query.QueryRequestImpl;
import com.agilecontrol.nea.core.query.QueryResult;
import com.agilecontrol.nea.core.query.QuerySession;
import com.agilecontrol.nea.core.query.web.SumFieldInterpreter;
import com.agilecontrol.nea.core.query.web.TableQueryModel;
import com.agilecontrol.nea.core.schema.Column;
import com.agilecontrol.nea.core.schema.Table;
import com.agilecontrol.nea.core.schema.TableManager;
import com.agilecontrol.nea.core.util.ConfigValues;
import com.agilecontrol.nea.util.Configurations;
import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Locale;
import javax.servlet.http.HttpServletRequest;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ReportUtils {

   private static final Logger log = LoggerFactory.getLogger(ReportUtils.class);
   String userName;
   UserWebImpl user = null;
   Configurations conf = null;
   Locale locale = null;


   public ReportUtils(HttpServletRequest request) {
      SessionContextManager mgr = WebUtils.getSessionContextManager(request.getSession(true));
      if(mgr != null) {
         this.user = (UserWebImpl)mgr.getActor("nea.web.authorization");
         this.locale = this.user.getLocale();
      }

      this.conf = (Configurations)WebUtils.getServletContextManager().getActor("nea.configs");
   }

   public String getUserName() {
      return this.user.getUserName();
   }

   public UserWebImpl getUser() {
      return this.user;
   }

   public String getExportRootPath() {
      return this.conf.getProperty("export.root.nds", "/aic/home");
   }

   public String getOracleExportRootPath() {
      return this.conf.getProperty("export.root.oracle", "/aic/home");
   }

   public void removeOldFiles() {
      File dir = new File(this.getExportRootPath() + File.separator + this.user.getClientDomain() + File.separator + this.user.getUserId());
      if(dir.exists() && dir.isDirectory()) {
         long keepTime = System.currentTimeMillis() - (long)(ConfigValues.get("export.keepfile.hours", 24) * 1000 * 3600);
         File[] files = dir.listFiles();

         for(int i = 0; i < files.length; ++i) {
            if(files[i].isFile() && files[i].lastModified() < keepTime && files[i].delete()) {
               log.debug("deleted {}", files[i].getAbsolutePath());
            }
         }

      }
   }

   public long getQuota() {
      String quota = this.conf.getProperty("export.quota", "5M");
      return quota.substring(quota.length() - 1).equalsIgnoreCase("m")?Integer.valueOf(quota.substring(0, quota.length() - 1)).longValue() * 1024L * 1024L:(quota.substring(quota.length() - 1).equalsIgnoreCase("k")?Integer.valueOf(quota.substring(0, quota.length() - 1)).longValue() * 1024L:Integer.valueOf(quota.substring(0, quota.length())).longValue());
   }

   public long getSpaceUsed() {
      File dir = new File(this.getExportRootPath() + File.separator + this.user.getClientDomain() + File.separator + this.user.getUserId());
      if(dir.exists() && dir.isDirectory()) {
         File[] files = dir.listFiles();
         long spaceUsed = 0L;

         for(int i = 0; i < files.length; ++i) {
            if(files[i].isFile()) {
               spaceUsed += files[i].length();
            }
         }

         return spaceUsed;
      } else {
         return 0L;
      }
   }

   public static String getHeader(boolean pk, boolean ak, String separator, QueryRequest qRequest) {
      String[] showNames = qRequest.getDisplayColumnNames(true);
      int[] showColumns = qRequest.getDisplayColumnIndices();
      if(separator == null || separator.trim().length() == 0) {
         separator = ",";
      }

      StringBuilder buff = new StringBuilder();

      for(int i = 0; i < showColumns.length; ++i) {
         int[] cLink = qRequest.getSelectionColumnLink(showColumns[i]);
         if((pk || cLink[cLink.length - 1] != qRequest.getMainTable().getPrimaryKey().getId()) && (ak || cLink[cLink.length - 1] != qRequest.getMainTable().getAlternateKey().getId())) {
            if(i > 0) {
               buff.append(separator);
            }

            buff.append(showNames[i]);
         }
      }

      return buff.toString();
   }

   public static String getHeaderHtml(boolean pk, boolean ak, QueryRequest qRequest) {
      String[] showNames = qRequest.getDisplayColumnNames(true);
      int[] showColumns = qRequest.getDisplayColumnIndices();
      String buff = "";
      buff = buff + "<tr>";

      for(int i = 0; i < showColumns.length; ++i) {
         int[] cLink = qRequest.getSelectionColumnLink(showColumns[i]);
         if((pk || cLink[cLink.length - 1] != qRequest.getMainTable().getPrimaryKey().getId()) && (ak || cLink[cLink.length - 1] != qRequest.getMainTable().getAlternateKey().getId())) {
            buff = buff + "<td>";
            buff = buff + showNames[i];
            buff = buff + "</td>";
         }
      }

      buff = buff + "</tr>";
      return buff;
   }

   public String getTitle(String sheet, String objectId, String separator) throws Exception {
      TableManager manager = TableManager.getInstance();
      int sheetId = Integer.parseInt(sheet);
      Table sheetTable = manager.getTable(sheetId);
      QueryRequestImpl query = QueryEngine.getInstance().createRequest((QuerySession)null);
      query.setMainTable(sheetId);
      query.addAllShowableColumnsToSelection(2);
      query.addParam(sheetTable.getPrimaryKey().getId(), objectId);
      QueryResult result = QueryEngine.getInstance().doQuery(query);
      if(result.getRowCount() == 0) {
         return "没有选中数据（" + sheet + "," + objectId + "）";
      } else if(result.getRowCount() == 1) {
         result.next();
         ArrayList columns = sheetTable.getShowableColumns(2);
         TableQueryModel model = new TableQueryModel(sheetId, 2, false, this.locale);
         String title = "";
         String data = "";

         Column col;
         for(int it = 0; it < columns.size(); ++it) {
            col = (Column)columns.get(it);
            if(!title.equals("")) {
               title = title + separator;
               data = data + separator;
            }

            title = title + model.getDescriptionForColumn(col);
            data = data + result.getString(it + 1, false);
         }

         SumFieldInterpreter sfi;
         for(Iterator var18 = sheetTable.getSumFields(); var18.hasNext(); data = data + sfi.parseValue(objectId + "")) {
            col = (Column)var18.next();
            String interpreter = col.getValueInterpeter();
            if(!title.equals("")) {
               title = title + separator;
               data = data + separator;
            }

            title = title + col.getDescription(TableManager.getDefaultSysLocale());
            int type = col.getSQLType();
            sfi = new SumFieldInterpreter(sheetTable.getName() + "Id", type, interpreter);
         }

         return title + "\n" + data + "\n";
      } else {
         return result.getRowCount() + "条数据（" + sheet + "," + objectId + "）";
      }
   }

   public String getTitleHtml(String sheet, String objectId) throws Exception {
      TableManager manager = TableManager.getInstance();
      int sheetId = Integer.parseInt(sheet);
      Table sheetTable = manager.getTable(sheetId);
      QueryRequestImpl query = QueryEngine.getInstance().createRequest((QuerySession)null);
      query.setMainTable(sheetId);
      query.addAllShowableColumnsToSelection(2);
      query.addParam(sheetTable.getPrimaryKey().getId(), objectId);
      QueryResult result = QueryEngine.getInstance().doQuery(query);
      if(result.getRowCount() == 0) {
         return "没有选中数据（" + sheet + "," + objectId + "）";
      } else if(result.getRowCount() != 1) {
         return result.getRowCount() + "条数据（" + sheet + "," + objectId + "）";
      } else {
         result.next();
         ArrayList columns = sheetTable.getShowableColumns(2);
         TableQueryModel model = new TableQueryModel(sheetId, 2, false, this.locale);
         String title = "<tr>";
         String data = "<tr>";

         Column col;
         for(int it = 0; it < columns.size(); ++it) {
            col = (Column)columns.get(it);
            title = title + "<td>" + model.getDescriptionForColumn(col) + "</td>";
            data = data + "<td>" + result.getString(it + 1, false) + "</td>";
         }

         SumFieldInterpreter sfi;
         for(Iterator var17 = sheetTable.getSumFields(); var17.hasNext(); data = data + "<td>" + sfi.parseValue(objectId + "") + "</td>") {
            col = (Column)var17.next();
            String interpreter = col.getValueInterpeter();
            title = title + "<td>" + col.getDescription(TableManager.getDefaultSysLocale()) + "</td>";
            int type = col.getSQLType();
            sfi = new SumFieldInterpreter(sheetTable.getName() + "Id", type, interpreter);
         }

         title = title + "</tr>";
         data = data + "</tr>";
         return title + "\n" + data + "\n";
      }
   }

}
